# -*- coding: utf-8 -*-

#    VOXplot
#    
#    Copyright: Jörg Mayer
#    Email: joerg.mayer@lingphon.com
#    Web: https://voxplot.lingphon.com
#    
#    VOXplot is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#    
#    VOXplot is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#    
#    You should have received a copy of the GNU General Public
#    License along with VOXplot (cf. COPYING). If not, see
#    https://www.gnu.org/licenses/gpl-3.0.en.html



import tkinter as tk
from tkinter import ttk
from tkinter.messagebox import showerror, showinfo
import urllib.request
import webbrowser
from datetime import datetime



class Window(tk.Toplevel):
	'''display license and copyright information etc.

	:param parent: Object, reference to parent window
	:param ASet: Object, reference to AppSettings instance (~global variables)
	:param i_lang: Int, index of selected interface language
	'''
	def __init__(self, parent, ASet, i_lang):
		super().__init__(parent)
		self.resizable(tk.FALSE,tk.FALSE)
		self.grab_set()
		self.attributes('-topmost', 1)
		self.protocol('WM_DELETE_WINDOW', self._close_pressed)
		self.aset = ASet
		_ = self.aset.translate

		self.aset.style.configure('Head.TLabel', font=self.aset.font_family + ' ' + str(self.aset.font_size + 2) + ' bold')

		ab_frame = ttk.Frame(self, padding=(75,10,75,10))
		ab_frame.grid(column=0, row=0)
		self.columnconfigure(0, weight=0)
		self.rowconfigure(0, weight=0)

		self.logoimg = tk.PhotoImage(file=self.aset.logo_s)
		self.logo = ttk.Label(ab_frame,
			text=self.aset.app_name,
			font=f'-family {self.aset.font_family} -size 18 -weight bold',
			image=self.logoimg,
			compound='top'
			)
		self.logo.grid(column=0, row=0)
		ttk.Label(
			ab_frame,
			text=_("– Acoustic Voice Quality Profile –"),
			# font='-weight bold',
			style='Head.TLabel',
			).grid(column=0, row=1, pady=(0,0))
		ttk.Label(
			ab_frame,
			text="(research use only)",
			font=f'-family {self.aset.font_family} -size 10 -slant italic'
			).grid(column=0, row=2, pady=(0,0))
		ttk.Label(
			ab_frame,
			text='v' + self.aset.app_version + ' (' + self.aset.app_date + ')',
			font=f'-family {self.aset.font_family} -size 10 -slant italic'
			).grid(column=0, row=3, pady=(0,10))

		ttk.Button(
			ab_frame,
			text=_("Check for Updates"),
			command=self._check_for_updates
			).grid(column=0, row=4, pady=(0,20))

		ttk.Label(
			ab_frame,
			text=_("Developer:"),
			# font='-weight bold',
			style='Head.TLabel',
			).grid(column=0, row=5, pady=(0,0))
		ttk.Label(
			ab_frame,
			text="Dr. Jörg Mayer / lingphon"
			).grid(column=0, row=6, pady=(0,0))
		link = tk.Label(
			ab_frame,
			text='voxplot.lingphon.com',
			fg='blue',
			cursor='hand2'
			)
		link.grid(column=0, row=7, pady=(0,10))
		if i_lang == 1:
			link.bind('<Button-1>', lambda e: self._open_url('https://voxplot.lingphon.com/de/'))
		else:
			link.bind('<Button-1>', lambda e: self._open_url('https://voxplot.lingphon.com/en/'))

		ttk.Label(
			ab_frame,
			text=_("Contributors:"),
			# font='-weight bold',
			style='Head.TLabel',
			).grid(column=0, row=8, pady=(0,0))
		ttk.Label(
			ab_frame,
			text="Mohamed Faska"
			).grid(column=0, row=9, pady=(0,10))

		ttk.Label(
			ab_frame,
			text=_("Scientific Consultant:"),
			# font='-weight bold',
			style='Head.TLabel',
			).grid(column=0, row=20, pady=(0,0))
		ttk.Label(
			ab_frame,
			text="Prof. Ben Barsties v. Latoszek"
			).grid(column=0, row=21, pady=(0,10))

		ttk.Label(
			ab_frame,
			text=_("Translators:"),
			# font='-weight bold',
			style='Head.TLabel',
			).grid(column=0, row=25, pady=(0,0))
		ttk.Label(
			ab_frame,
			text=("Jörg Mayer (de, en)\n"
				"Rudi Verfaillie (nl)"
				)
			).grid(column=0, row=26, pady=(0,20))

		ttk.Label(
			ab_frame,
			text=_("License"),
			# font='-weight bold',
			style='Head.TLabel',
			).grid(column=0, row=30, pady=(0,0))
		gpllink = tk.Label(
			ab_frame,
			text='GNU General Public License, Version 3',
			fg='blue',
			cursor='hand2'
			)
		gpllink.grid(column=0, row=31, pady=(0,0))
		gpllink.bind('<Button-1>', lambda e: self._open_url('https://www.gnu.org/licenses/gpl-3.0.html'))
		now = datetime.now()
		yearstring = ('2021-' + str(now.year) if now.year > 2021 else '2021')
		ttk.Label(
			ab_frame,
			text='Copyright ' + yearstring + ' Jörg Mayer'
			).grid(column=0, row=32, pady=(0,20))

		btn_frame = ttk.Frame(ab_frame)
		btn_frame.grid(column=0, row=40, pady=(0,0), sticky='we')
		btn_frame.columnconfigure(0, weight=1)
		ttk.Button(
			btn_frame,
			text=_("Close"),
			command=self._close_pressed
			).grid(column=0, row=0, sticky='sw')
		ttk.Button(
			btn_frame,
			text=_("Donate"),
			command=self._donate
			).grid(column=1, row=0, sticky='se')

		self.bind('<KeyPress-Escape>', self._close_pressed)


	def _close_pressed(self, *args):
		self.destroy()


	def _open_url(self, url):
		webbrowser.open_new(url)


	def _donate(self):
		webbrowser.open_new('https://www.paypal.com/donate/?hosted_button_id=9SYWMK8RSAZD4')
		self.destroy()


	def _check_for_updates(self, *args):
		_ = self.aset.translate
		try:
			update_info = urllib.request.urlopen('https://www.lingphonserver.de/voxplotversion')
		except:
			showerror(
				_("Update check failed"),
				_("Update info is currently not available. Please try again later."),
				parent=self
				)
			return
		server_version, server_date = update_info.read().decode('utf-8').split(',')
		major_s, minor_s, patch_s = server_version.split('.')
		major_a, minor_a, patch_s = self.aset.app_version.split('.')
		if not patch_s.isnumeric():
			patch_s, release_s = patch_s.split(' ', 1)
		if not patch_a.isnumeric():
			patch_a, release_a = patch_a.split(' ', 1)

		if int(major_s) > int(major_a) or (int(major_s) == int(major_a) and int(minor_s) > int(minor_a)) or (int(major_s) == int(major_a) and int(minor_s) == int(minor_a) and int(patch_s) > int(patch_s)):
			showinfo(
				_("Update check"),
				_("A new version of VOXplot is available") + ' (v' + server_version + '). ' +\
				_("To download the new version visit") + (' voxplot.lingphon.com'),
				parent=self
				)
		else:
			showinfo(
				_("Update check"),
				_("Your version of VOXplot is up to date. Nothing to do."),
				parent=self
				)





