# -*- coding: utf-8 -*-

#    VOXplot
#    
#    Copyright: Jörg Mayer
#    Email: joerg.mayer@lingphon.com
#    Web: https://voxplot.lingphon.com
#    
#    VOXplot is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#    
#    VOXplot is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#    
#    You should have received a copy of the GNU General Public
#    License along with VOXplot (cf. COPYING). If not, see
#    https://www.gnu.org/licenses/gpl-3.0.en.html



import tkinter as tk
from tkinter import ttk
import os



class Window(tk.Toplevel):
	'''display information about analysis language

	:param parent: Object, reference to parent window
	:param ASet: Object, reference to AppSettings instance (~global variables)
	:param a_lang: Int, index of selected analysis language
	'''
	def __init__(self, parent, ASet, a_lang):
		super().__init__(parent)
		self.resizable(tk.FALSE,tk.FALSE)
		# self.grab_set()
		self.attributes('-topmost', 1)
		self.protocol('WM_DELETE_WINDOW', self._close_pressed)
		self.aset = ASet
		_ = self.aset.translate

		al_frame = ttk.Frame(self, padding=(75,10,75,10))
		al_frame.grid(column=0, row=0)
		self.columnconfigure(0, weight=0)
		self.rowconfigure(0, weight=0)

		ttk.Label(
			al_frame,
			text=_("Analysis language") + ': ' + self.aset.analysis_langs[a_lang],
			font=f'-family {self.aset.font_family} -weight bold'
			).grid(column=0, row=0, pady=(0,20))

		if self.aset.cs_text.rfind("png") >= 0:
			# cs_text is transparent png image (h = 140 px, w < 650px)
			self.cs_text_img = tk.PhotoImage(file=os.path.join(self.aset.data_dir, self.aset.cs_text))
			ttk.Label(
				al_frame,
				image=self.cs_text_img,
				).grid(column=0, row=1)
		else:
			# cs_text is text
			ttk.Label(
				al_frame,
				text=self.aset.cs_text,
				font=f'-family Times -size 16 -slant italic'
				).grid(column=0, row=1)

		btn_frame = ttk.Frame(al_frame)
		btn_frame.grid(column=0, row=20, pady=(20,0), sticky='we')
		btn_frame.columnconfigure(0, weight=1)
		ttk.Button(
			btn_frame,
			text=_("Close"),
			command=self._close_pressed
			).grid(column=0, row=0)

		self.bind('<KeyPress-Escape>', self._close_pressed)


	def _close_pressed(self, *args):
		self.destroy()


