# -*- coding: utf-8 -*-

#    VOXplot
#    
#    Copyright: Jörg Mayer
#    Email: joerg.mayer@lingphon.com
#    Web: https://voxplot.lingphon.com
#    
#    VOXplot is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#    
#    VOXplot is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#    
#    You should have received a copy of the GNU General Public
#    License along with VOXplot (cf. COPYING). If not, see
#    https://www.gnu.org/licenses/gpl-3.0.en.html


import tkinter as tk
from tkinter import ttk, filedialog
from tkinter.messagebox import showerror, showwarning, showinfo, askokcancel
from tkcalendar import Calendar, DateEntry
import threading
import os
import wave
import pyaudio
from pydub import AudioSegment
from datetime import datetime
from shutil import copyfile
from voxplot import about, donation, settings, record, checksnr, analysis, signalplot, alanginfo #, corpus



class Window:
	'''The main window.

	:param root: Object, reference to root window
	:param ASet: Object, reference to AppSettings instance (~global variables)
	:param USet: Object, reference to UserSettings instance
	'''
	def __init__(self, root, ASet, USet):
		self.root = root
		self.root.resizable(tk.FALSE,tk.FALSE)
		self.root.columnconfigure(0, weight=1)
		self.root.rowconfigure(0, weight=1)
		self.aset = ASet
		self.uset = USet
		_ = self.aset.translate
		self._clear_data()

		###frames
		mainframe = ttk.Frame(self.root, padding=(10,10,10,10))
		mainframe.grid(column=0, row=0, sticky='nswe')
		mainframe.columnconfigure(0, weight=1)
		mainframe.columnconfigure(1, weight=1)
		mainframe.columnconfigure(2, weight=1)
		mainframe.rowconfigure(0, weight=1)
		mainframe.rowconfigure(1, weight=1)

		topframe = ttk.Frame(mainframe)
		topframe.grid(column=0, row=0, columnspan=3, sticky='we', pady=(10,0))
		topframe.columnconfigure(0, weight=1)
		topframe.columnconfigure(1, weight=0)
		topframe.columnconfigure(2, weight=0)
		topframe.columnconfigure(3, weight=0)

		detailsframe = ttk.Frame(mainframe)
		detailsframe.grid(column=0, row=1, sticky='ns', pady=(60,0))
		detailsframe.rowconfigure(0, weight=0)
		detailsframe.rowconfigure(1, weight=0)
		detailsframe.rowconfigure(2, weight=0)
		detailsframe.rowconfigure(3, weight=0)
		detailsframe.rowconfigure(4, weight=0)
		detailsframe.rowconfigure(5, weight=0)
		detailsframe.rowconfigure(6, weight=0)
		detailsframe.rowconfigure(7, weight=0)
		detailsframe.rowconfigure(8, weight=0)
		detailsframe.rowconfigure(9, weight=0)
		detailsframe.rowconfigure(10, weight=0)
		detailsframe.rowconfigure(11, weight=1)

		separatorframe = ttk.Frame(mainframe, relief=tk.RIDGE)
		separatorframe.grid(column=1, row=1, padx=20, sticky='ns')
		ttk.Separator(separatorframe, orient=tk.VERTICAL).grid(column=0, row=0)

		samplesframe = ttk.Frame(mainframe)
		samplesframe.grid(column=2, row=1, sticky='n')
		samplesframe.columnconfigure(0, weight=0)
		samplesframe.columnconfigure(1, weight=0)
		samplesframe.columnconfigure(2, weight=0)
		samplesframe.columnconfigure(3, weight=0)
		samplesframe.columnconfigure(4, weight=1)
		samplesframe.columnconfigure(5, weight=0)

		### styles
		self.aset.style.configure('Smpl.TButton', font=self.aset.font_family + ' ' + str(self.aset.font_size - 2))
		self.aset.style.configure('Head.TLabel', font=self.aset.font_family + ' ' + str(self.aset.font_size + 4) + ' bold')


		### top frame widgets
		self.logoimg = tk.PhotoImage(file=self.aset.logo_s).subsample(2)
		self.logo = ttk.Label(
			topframe,
			text=self.aset.app_name,
			font=f'-family {self.aset.font_family} -size 24 -weight bold',
			image=self.logoimg,
			compound='left'
			)
		self.logo.grid(column=0, row=0, sticky='w')

		ttk.Button(
			topframe,
			text=_("About"),
			command=self._about_pressed
			).grid(column=1, row=0, sticky='ne', pady=(0,0), padx=(0,5))
		ttk.Button(
			topframe,
			text=_("Settings"),
			command=self._settings_pressed
			).grid(column=2, row=0, sticky='ne', pady=(0,0), padx=(0,5))
		ttk.Button(
			topframe,
			text=_("Quit"),
			# style='Quit.TButton',
			command=self._quit_pressed
			).grid(column=3, row=0, sticky='ne', pady=(0,0))


		### detailsframe widgets
		ttk.Label(
			detailsframe,
			text=_("Subject"),
			style='Head.TLabel',
			# font='-size 14 -weight bold'
			).grid(column=0, row=0, columnspan=4, sticky='w', pady=(0,10))

		ttk.Label(
			detailsframe,
			text=_("Name")
			).grid(column=0, row=1, sticky='ne', padx=(5,10), pady=(10,0))
		self.subject_name = tk.StringVar()
		subject_name_entry = ttk.Entry(
			detailsframe,
			width=20,
			textvariable=self.subject_name
			)
		subject_name_entry.grid(column=1, row=1, columnspan=3, sticky='sw')

		ttk.Label(
			detailsframe,
			text=_("ID")
			).grid(column=0, row=2, sticky='ne', padx=(5,10), pady=(10,0))
		self.subject_id = tk.StringVar()
		subject_id_entry = ttk.Entry(
			detailsframe,
			width=20,
			textvariable=self.subject_id
			)
		subject_id_entry.grid(column=1, row=2, columnspan=3, sticky='sw')

		ttk.Label(
			detailsframe,
			text=_("Title")
			).grid(column=0, row=3, sticky='e', padx=(5,10), pady=(10,0))
		self.sex = tk.StringVar()
		genderframe = ttk.Frame(detailsframe)
		genderframe.grid(column=1, columnspan=3, row=3, sticky='w')
		ttk.Radiobutton(
			genderframe,
			text=_("Ms."),
			variable=self.sex,
			value='f'
			).grid(column=0, row=0, pady=(10,0), padx=(0,10), sticky='w')
		ttk.Radiobutton(
			genderframe,
			text=_("Mr."),
			variable=self.sex,
			value='m'
			).grid(column=1, row=0, pady=(10,0), padx=(0,10), sticky='w')
		self.default_sex = ttk.Radiobutton(
			genderframe,
			text=_("w/o"),
			variable=self.sex,
			value='x'
			)
		self.default_sex.grid(column=2, row=0, pady=(10,0), padx=(0,10), sticky='w')
		self.default_sex.invoke()

		ttk.Label(
			detailsframe,
			text=_("Date of Birth")
			).grid(column=0, row=4, sticky='ne', padx=(5,10), pady=(10,0))
		self.subject_dob = tk.StringVar()
		subject_dob_entry = ttk.Entry(
			detailsframe,
			width=10,
			textvariable=self.subject_dob
			)
		subject_dob_entry.grid(column=1, row=4, sticky='sw')

		ttk.Label(
			detailsframe,
			text=_("Assessment/Notes")
			).grid(column=0, row=5, sticky='ne', padx=(5,10), pady=(10,0))
		self.subject_notes = tk.Text(
			detailsframe,
			width=20,
			height=5,
			wrap='word',
			font='TkDefaultFont',
			relief='ridge'
			)
		self.subject_notes.grid(column=1, row=5, columnspan=3, sticky='nw', pady=(10,0))

		ttk.Label(
			detailsframe,
			text=_("Examination"),
			style='Head.TLabel',
			# font='-size 14 -weight bold'
			).grid(column=0, row=6, columnspan=4, sticky='w', pady=(30,10))
 
		ttk.Label(
			detailsframe,
			text=_("Examiner")
			).grid(column=0, row=7, sticky='ne', padx=(5,10), pady=(10,0))
		self.exam_name = tk.StringVar()
		exam_name_entry = ttk.Entry(
			detailsframe,
			width=20,
			textvariable=self.exam_name
			)
		self.exam_name.set(self.uset.examiner)
		exam_name_entry.grid(column=1, row=7, columnspan=3,sticky='sw')

		ttk.Label(
			detailsframe,
			text=_("Date")
			).grid(column=0, row=8, sticky='ne', padx=(5,10), pady=(10,0))
		self.exam_date = DateEntry(
			detailsframe,
			width=10
			)
		self.exam_date.grid(column=1, row=8, sticky='sw')
		self.exam_date.set_date(datetime.today())
		self.set_date_format(self.exam_date)

		ttk.Label(
			detailsframe,
			text=_("Time")
			).grid(column=0, row=9, sticky='ne', padx=(5,10), pady=(10,0))
		self.exam_time = tk.StringVar()
		exam_time_entry = ttk.Entry(
			detailsframe,
			width=10,
			textvariable=self.exam_time
			)
		exam_time_entry.grid(column=1, row=9, sticky='sw')
		self.exam_time.set(self.format_time(datetime.today()))

		ttk.Button(
			detailsframe,
			text=_("Clear data and samples"),
			command=lambda: self.clear_pressed(clearall=True)
			).grid(column=0, columnspan=2, row=11, sticky='sw', padx=(0,0))


		### samplesframe widgets: cs
		ttk.Label(
			samplesframe,
			text=_("Continuous Speech"),
			style='Head.TLabel',
			# font='-size 14 -weight bold'
			).grid(column=0, row=0, columnspan=3, sticky='w', pady=(0,5))

		self.alanginfo_link = ttk.Label(
			samplesframe,
			text=_("Analysis language") + ': ' + self.aset.analysis_langs[self.uset.a_lang],
			cursor='hand2',
			foreground='blue',
			font=f'-family {self.aset.font_family} -size {str(self.aset.font_size - 1)}'
			)
		self.alanginfo_link.grid(column=3, row=0, columnspan=3, sticky='ne', pady=(0,5))
		self.alanginfo_link.bind('<Button-1>', self._alanginfo_pressed)

		self.cs_open_btn = ttk.Button(
			samplesframe,
			text=_("Open"),
			style='Smpl.TButton',
			width=9,
			command=lambda: self._open_pressed('cs')
			)
		self.cs_open_btn.grid(column=0, row=1, sticky='sw', padx=(0,5))

		self.cs_new_btn = ttk.Button(
			samplesframe,
			text=_("New"),
			style='Smpl.TButton',
			width=9,
			command=lambda: self._new_pressed('cs')
			)
		self.cs_new_btn.grid(column=1, row=1, sticky='sw', padx=(0,5))

		self.cs_play_btn = ttk.Button(
			samplesframe,
			text=_("Play"),
			style='Smpl.TButton',
			width=9,
			command=lambda: self._start_playing(self.aset.cs_file)
			)
		self.cs_play_btn.grid(column=2, row=1, sticky='sw', padx=(0,5))

		self.cs_edit_btn = ttk.Button(
			samplesframe,
			text=_("Edit"),
			style='Smpl.TButton',
			width=9,
			command=lambda: self._edit_pressed('cs')
			)
		self.cs_edit_btn.grid(column=3, row=1, sticky='sw', padx=(0,5))

		self.cs_save_btn = ttk.Button(
			samplesframe,
			text=_("Save"),
			style='Smpl.TButton',
			width=9,
			command=lambda: self._save_pressed('cs')
			)
		self.cs_save_btn.grid(column=5, row=1, sticky='se')

		main_cs_box = ttk.Frame(samplesframe)
		main_cs_box.grid(column=0, row=2, columnspan=6, sticky='we', pady=(5,3))

		self.cs_filename = tk.StringVar()
		self.cs_filename.set("")
		self.cs_filename_entry = tk.Entry(
			samplesframe,
			width=int(self.aset.font_size * 3.5),
			textvariable=self.cs_filename,
			state="disabled",
			font=f'-family {self.aset.font_family} -size {str(self.aset.font_size)}',
			bg=self.root.cget('bg'),
			disabledforeground='',
			disabledbackground='',
			readonlybackground='',
			borderwidth=0,
			relief='flat',
			)
		self.cs_filename_entry.grid(column=0, row=3, columnspan=4, sticky='w')

		ttk.Label(
			samplesframe,
			text=_("Recording date:") + ' ',
			font=f'-family {self.aset.font_family} -size {str(self.aset.font_size - 2)}',
			).grid(column=4, row=3, sticky='e', pady=(1,0))
		self.cs_date_entry = DateEntry(samplesframe, width=10)
		self.cs_date_entry.grid(column=5, row=3, sticky='e')
		self.cs_date_entry.set_date(datetime.today())
		self.set_date_format(self.cs_date_entry)


		### samplesframe widgets: sv
		ttk.Label(
			samplesframe,
			text=_("Sustained Vowel"),
			style='Head.TLabel',
			# font='-size 14 -weight bold'
			).grid(column=0, row=10, columnspan=6, sticky='sw', pady=(10,5))

		self.sv_open_btn = ttk.Button(
			samplesframe,
			text=_("Open"),
			style='Smpl.TButton',
			width=9,
			command=lambda: self._open_pressed('sv')
			)
		self.sv_open_btn.grid(column=0, row=11, sticky='sw', padx=(0,5))

		self.sv_new_btn = ttk.Button(
			samplesframe,
			text=_("New"),
			style='Smpl.TButton',
			width=9,
			command=lambda: self._new_pressed('sv')
			)
		self.sv_new_btn.grid(column=1, row=11, sticky='sw', padx=(0,5))

		self.sv_play_btn = ttk.Button(
			samplesframe,
			text=_("Play"),
			style='Smpl.TButton',
			width=9,
			command=lambda: self._start_playing(self.aset.sv_file)
			)
		self.sv_play_btn.grid(column=2, row=11, sticky='sw', padx=(0,5))

		self.sv_edit_btn = ttk.Button(
			samplesframe,
			text=_("Edit"),
			style='Smpl.TButton',
			width=9,
			command=lambda: self._edit_pressed('sv')
			)
		self.sv_edit_btn.grid(column=3, row=11, sticky='sw', padx=(0,5))

		self.sv_save_btn = ttk.Button(
			samplesframe,
			text=_("Save"),
			style='Smpl.TButton',
			width=9,
			command=lambda: self._save_pressed('sv')
			)
		self.sv_save_btn.grid(column=5, row=11, sticky='se')

		main_sv_box = ttk.Frame(samplesframe)
		main_sv_box.grid(column=0, row=12, columnspan=6, sticky='we', pady=(5,3))

		self.sv_filename = tk.StringVar()
		self.sv_filename.set("")
		self.sv_filename_entry = tk.Entry(
			samplesframe,
			width=int(self.aset.font_size * 3.5),
			textvariable=self.sv_filename,
			state="disabled",
			font=f'-family {self.aset.font_family} -size {str(self.aset.font_size)}',
			bg=self.root.cget('bg'),
			disabledforeground='',
			disabledbackground='',
			readonlybackground='',
			borderwidth=0,
			relief='flat',
			)
		self.sv_filename_entry.grid(column=0, row=13, columnspan=4, sticky='w')

		ttk.Label(
			samplesframe,
			text=_("Recording date:") + ' ',
			font=f'-family {self.aset.font_family} -size {str(self.aset.font_size - 2)}',
			).grid(column=4, row=13, sticky='e', pady=(1,0))
		self.sv_date_entry = DateEntry(samplesframe, width=10)
		self.sv_date_entry.grid(column=5, row=13, sticky='e')
		self.sv_date_entry.set_date(datetime.today())
		self.set_date_format(self.sv_date_entry)


		### samplesframe widgets: misc
		ttk.Button(
			samplesframe,
			text=_("Clear samples"),
			command=lambda: self.clear_pressed(clearall=False)
			).grid(column=0, row=20, columnspan=2, sticky='nw', pady=(20,0))

		ttk.Button(
			samplesframe,
			text=_("Check SNR"),
			command=self._check_snr
			).grid(column=2, row=20, columnspan=2, sticky='nw', pady=(20,0), padx=(10,0))

		self.aset.style.configure('Ana.TButton', foreground='white', background='green')
		self.aset.style.map(
			'Ana.TButton',
			background=[('disabled','#d9d9d9'), ('active','#009C00')],
			foreground=[('disabled','#a3a3a3'), ('active', '#fff')]
			)

		### corpus analysis is still just an idea, we don't yet activate it
		'''
		self.corpus_btn = ttk.Button(
			samplesframe,
			text=_("Corpus analysis"),
			style='Ana.TButton',
			command=self._start_corpus_analysis
			)
		self.corpus_btn.grid(column=2, row=20, columnspan=2, sticky='nw', pady=(20,0), ipadx=25)
		'''

		self.analyze_btn = ttk.Button(
			samplesframe,
			text=_("Voice Quality Analysis"),
			style='Ana.TButton',
			command=self._start_analysis
			)
		self.analyze_btn.grid(column=4, row=20, columnspan=2, sticky='ne', pady=(20,0), ipadx=25)


		### bindings etc
		subject_name_entry.focus()
		# call quit function if root window is closed
		self.root.protocol('WM_DELETE_WINDOW', self._quit_pressed)
		# Example: bind some function to Return key pressed in self.root window
		# self.root.bind("<Return>", self.some_function)


		### initializations
		# plots
		self.main_cs_plot = signalplot.Plotter('cs', main_cs_box, self.aset, self.uset)
		self.main_cs_plot.init_plot(recdur_factor=1, reset=False)
		self.main_sv_plot = signalplot.Plotter('sv', main_sv_box, self.aset, self.uset)
		self.main_sv_plot.init_plot(recdur_factor=1, reset=False)

		# root window positioning
		self.root.update_idletasks()
		windowWidth = self.root.winfo_reqwidth()
		windowHeight = self.root.winfo_reqheight()
		xposition = int(self.root.winfo_screenwidth()/2 - windowWidth/2)
		yposition = int(self.root.winfo_screenheight()/2 - windowHeight/2)
		self.root.geometry("+{}+{}".format(xposition, yposition))

		# button states
		self.cs_play_btn.state(['disabled'])
		self.cs_edit_btn.state(['disabled'])
		self.cs_date_entry.state(['disabled'])
		self.cs_save_btn.state(['disabled'])
		self.sv_play_btn.state(['disabled'])
		self.sv_edit_btn.state(['disabled'])
		self.sv_date_entry.state(['disabled'])
		self.sv_save_btn.state(['disabled'])
		self.analyze_btn.state(['disabled'])
		if not self.aset.input_device_available:
			self.cs_new_btn.state(['disabled'])
			self.sv_new_btn.state(['disabled'])

		# some variables
		self.cs_date_str = None
		self.rec_modal = None
		self.play_thread = None
		self.play_exit_event = threading.Event()
		self.play_target = None

		# launch settings window if no settings available
		if self.aset.no_settings:
			self._settings_pressed()
			donation.Window(self.root, self.aset)


	def clear_pressed(self, clearall=True):
		# reset application settings
		self.aset.sampling_rate = self.aset.default_sampling_rate
		self.aset.v_frames_analyzed = self.aset.v_seconds_analyzed * self.aset.sampling_rate
		self.aset.sv_too_short = False
		self.aset.sv_too_long = False

		# reset plots and filename display
		self.main_cs_plot.init_plot()
		self.main_cs_plot.figure.canvas.draw()
		self.main_sv_plot.init_plot()
		self.main_sv_plot.figure.canvas.draw()
		self.cs_filename_entry.configure(state="disabled")
		self.cs_filename.set("")
		self.sv_filename_entry.configure(state="disabled")
		self.sv_filename.set("")

		# disable buttons
		self.cs_play_btn.state(['disabled'])
		self.cs_edit_btn.state(['disabled'])
		self.cs_date_entry.state(['disabled'])
		self.cs_save_btn.state(['disabled'])
		self.sv_play_btn.state(['disabled'])
		self.sv_edit_btn.state(['disabled'])
		self.sv_date_entry.state(['disabled'])
		self.sv_save_btn.state(['disabled'])
		self.analyze_btn.state(['disabled'])
		# self.corpus_btn.state(['!disabled'])

		self._clear_data()

		if clearall:
			# reset metadata
			self.subject_name.set('')
			self.subject_id.set('')
			self.default_sex.invoke()
			self.subject_dob.set('')
			self.subject_notes.delete('1.0', 'end')
			self.exam_name.set(self.uset.examiner)
			self.exam_date.set_date(datetime.today())
			self.exam_time.set(self.format_time(datetime.today()))


	def _about_pressed(self, *args):
		about.Window(self.root, self.aset, self.uset.i_lang)


	def _settings_pressed(self, *args):
		settings.Window(self.root, self.aset, self.uset)


	def _quit_pressed(self, *args):
		self._clear_data()
		self.root.destroy()


	def _alanginfo_pressed(self, *args):
		alanginfo.Window(self.root, self.aset, self.uset.a_lang)


	def _open_pressed(self, type):
		_ = self.aset.translate
		if type == "cs": dialog_title = _("Choose a WAV file (continuous speech)")
		if type == "sv": dialog_title = _("Choose a WAV file (sustained vowel)")
		response = filedialog.askopenfilename(
			title = dialog_title,
			filetypes = (('WAV files', '.wav'), ('WAV files', '.WAV'))
			)
		if response != () and response != "":
			### the following button activation is superfluous, isn't ist?
			# self.analyze_btn.state(['!disabled'])

			# test if soundfile is 16bit
			testbit = wave.open(response, 'r')
			bit = testbit.getsampwidth() * 8
			testbit.close()
			# if not 16bit convert by read/save in Praat
			if bit != 16:
				import subprocess
				try:
					subprocess.run([self.aset.praat_exe, '--run', '--no-pref-files', '--no-plugins', '--utf8', self.aset.conv_script, response, self.aset.conv_tmp_file], check=True)
				except subprocess.CalledProcessError as e:
					return "A fatal error occurred during conversion."
				response = self.aset.conv_tmp_file
			sound = AudioSegment.from_wav(response)
			duration = sound.duration_seconds
			if sound.frame_rate < 22050:
				showerror(_("Error"),
					_("A sampling frequency of at least 22050 Hz is required. The selected sound file has only ") + str(sound.frame_rate) + ' Hz')
				return
			self.aset.sampling_rate = sound.frame_rate
			self.aset.v_frames_analyzed = self.aset.v_seconds_analyzed * self.aset.sampling_rate
			filename = os.path.basename(response)
			if sound.channels > 1:
				# NOTE: convert stereo to mono; may result in loss of quality if left and right channels differ.
				mono = sound.set_channels(1)
				mono.export(self.aset.response_tmp_file, format='wav')
				response = self.aset.response_tmp_file
			if type == "cs":
				copyfile(response, self.aset.cs_file)
				self.main_cs_plot.plot_sound(self.aset.cs_file)
				self.cs_play_btn.state(['!disabled'])
				self.cs_edit_btn.state(['!disabled'])
				self.cs_date_entry.state(['!disabled'])
				self.cs_save_btn.state(['!disabled'])
				self.analyze_btn.state(['!disabled'])
				# self.corpus_btn.state(['disabled'])
				self.update_sample_date(response, 'cs')
				self.cs_filename_entry.configure(state="readonly")
				self.cs_filename.set(filename)
				self._edit_pressed('cs')
			elif type == "sv":
				self.aset.sv_too_short = False
				self.aset.sv_too_long = False
				copyfile(response, self.aset.sv_file)
				self.main_sv_plot.plot_sound(self.aset.sv_file)
				self.sv_play_btn.state(['!disabled'])
				self.sv_edit_btn.state(['!disabled'])
				self.sv_date_entry.state(['!disabled'])
				self.sv_save_btn.state(['!disabled'])
				self.analyze_btn.state(['!disabled'])
				# self.corpus_btn.state(['disabled'])
				self.update_sample_date(response, 'sv')
				self.sv_filename_entry.configure(state="readonly")
				self.sv_filename.set(filename)
				# assert vowel duration between 2.95 and 3.05
				if duration < self.aset.v_seconds_analyzed - 0.05:
					self.aset.sv_too_short = True
					showwarning(_("Warning"),
						_("This sample is too short") + \
						' (' + str(duration) + 's)\n' + \
						_("AVQI / ABI analysis requires vowel duration = 3 seconds."))
				elif duration > self.aset.v_seconds_analyzed + 0.05:
					self.aset.sv_too_long = True
					self._edit_pressed('sv')


	def _new_pressed(self, type):
		self.aset.sampling_rate = self.aset.default_sampling_rate
		self.aset.v_frames_analyzed = self.aset.v_seconds_analyzed * self.aset.sampling_rate
		self.aset.rec_win = record.Window(self.root, self.aset, self.uset, type)


	def _start_playing(self, play_target):
		self.play_exit_event.clear()
		self.play_target = play_target
		self.root.unbind('<KeyPress-Escape>')
		self.root.bind('<KeyPress-Escape>', self._stop_playing)
		self.play_thread = threading.Thread(target=self._player)
		self.play_thread.start()


	def _stop_playing(self, *args):
		self.play_exit_event.set()


	def _after_playing(self):
		self.play_thread = None
		self.root.unbind('<KeyPress-Escape>')


	def _player(self):
		fname = self.play_target
		chunk = self.aset.chunk
		wavefile = wave.open(fname, 'rb')
		pa = pyaudio.PyAudio()
		stream = pa.open(
			format=pa.get_format_from_width(wavefile.getsampwidth()),
			channels=wavefile.getnchannels(),
			rate=wavefile.getframerate(),
			output=True
			)
		data = wavefile.readframes(chunk)
		while len(data) > 0 and not self.play_exit_event.is_set():
			stream.write(data)
			data = wavefile.readframes(chunk)
		wavefile.close()
		stream.stop_stream()
		stream.close()
		pa.terminate()
		self._after_playing()


	def _check_snr(self):
		self.aset.sampling_rate = self.aset.default_sampling_rate
		self.aset.v_frames_analyzed = self.aset.v_seconds_analyzed * self.aset.sampling_rate
		self.aset.snr_win = checksnr.Window(self.root, self.aset, self.uset)


	def _edit_pressed(self, type):
		if type == 'cs':
			copyfile(self.aset.cs_file, self.aset.tmp_file)
		elif type == 'sv':
			copyfile(self.aset.sv_file, self.aset.tmp_file)
		self.aset.rec_win = record.Window(self.root, self.aset, self.uset, type, is_edit=True)
		self.aset.rec_win.edit_sound()


	def _save_pressed(self, type):
		_ = self.aset.translate
		if (type == 'cs' and os.path.exists(self.aset.cs_file) or (type == 'sv' and os.path.exists(self.aset.sv_file))):
			if type == "cs": dialog_title = _("Save as WAV file (continuous speech)")
			if type == "sv": dialog_title = _("Save as WAV file (sustained vowel)")
			response = filedialog.asksaveasfilename(
				title=dialog_title,
				defaultextension='.wav',
				filetypes=(('WAV files', '*.wav'),),
				confirmoverwrite=True
				)
			if response != () and response != "":
				filename = os.path.basename(response)
				if type == "cs":
					sound = AudioSegment.from_wav(self.aset.cs_file)
					sound.export(response, format='wav')
					self.cs_filename_entry.configure(state="readonly")
					self.cs_filename.set(filename)
				elif type == "sv":
					sound = AudioSegment.from_wav(self.aset.sv_file)
					sound.export(response, format='wav')
					self.sv_filename_entry.configure(state="readonly")
					self.sv_filename.set(filename)
		else:
			return

	### corpus analysis is still just an idea, we don't yet activate it
	'''
	def _start_corpus_analysis(self):
		corpus.Window(self.root, self.aset, self.uset)
	'''


	def _start_analysis(self):
		_ = self.aset.translate
		indices = 0
		sv_ok = 0
		metadata = {}
		if not os.path.exists(self.aset.sv_file):
			okay = askokcancel(
				_("Voice sample missing"),
				_("Complete analysis requires both continuous speech sample and vowel sample.") +\
				'\n\n' + _("Voice quality indices (AVQI/ABI) can't be computed without vowel sample."),
				icon='warning'
				)
			if not okay: return
			samples = 'cs'
		elif not os.path.exists(self.aset.cs_file):
			okay = askokcancel(
				_("Voice sample missing"),
				_("Complete analysis requires both continuous speech sample and vowel sample.") +\
				'\n\n' + _("Voice quality indices (AVQI/ABI) can't be computed without continuous speech sample."),
				icon='warning'
				)
			if not okay: return
			samples = 'sv'
			sv_ok = self._test_sv('sv')
			if sv_ok == -1: return
		else:
			samples = 'mx'
			sv_ok = self._test_sv('mx')
			if sv_ok == -1:
				return
			elif sv_ok == 1:
				indices = 1
				sound = AudioSegment.from_wav(self.aset.sv_file)
				svsf = sound.frame_rate
				sound = AudioSegment.from_wav(self.aset.cs_file)
				cssf = sound.frame_rate
				if cssf > svsf:
					okay = askokcancel(
						"Resampling",
						_("Due to different sampling rates the continuous speech sample will be down-sampled for parts of the analysis."),
						icon='warning'
						)
					if not okay: return
				elif cssf < svsf:
					okay = askokcancel(
						"Resampling",
						_("Due to different sampling rates the vowel sample will be down-sampled for parts of the analysis."),
						icon='warning'
						)
					if not okay: return
		metadata['name'] = self.subject_name.get()
		metadata['subid'] = self.subject_id.get()
		metadata['sex'] = self.sex.get()
		metadata['dob'] = self.subject_dob.get()
		metadata['notes'] = self.subject_notes.get('1.0', 'end')
		metadata['examiner'] = self.exam_name.get()
		try:
			date = self.exam_date.get_date()
			metadata['date'] = self._format_date(date)
		except:
			metadata['date'] = ""
		metadata['time'] = self.exam_time.get()
		try:
			csdate = self.cs_date_entry.get_date()
			metadata['csdate'] = self._format_date(csdate)
		except:
			metadata['csdate'] = ""
		try:
			svdate = self.sv_date_entry.get_date()
			metadata['svdate'] = self._format_date(svdate)
		except:
			metadata['svdate'] = ""
		analysis.Window(
			self.root,
			self.aset,
			self.uset,
			samples,
			indices,
			sv_ok,
			metadata
			)


	### helper methods
	def set_date_format(self, date_field):
		if self.uset.i_lang == 0:
			date_field.configure(locale='en_US')
		elif self.uset.i_lang == 1:
			date_field.configure(locale='de_DE')
		elif self.uset.i_lang == 2:
			date_field.configure(locale='nl_NL')
		if self.uset.date_format == 0:
			date_field.configure(date_pattern='mm/dd/yyyy')
		elif self.uset.date_format == 1:
			date_field.configure(date_pattern='dd.mm.yyyy')
		elif self.uset.date_format == 2:
			date_field.configure(date_pattern='yyyy-mm-dd')


	def format_time(self, date_obj):
		if self.uset.time_format == 0:
			ap = (" pm" if date_obj.hour >= 12 else " am")
			return date_obj.strftime('%I:%M') + ap
		elif self.uset.time_format == 1:
			return date_obj.strftime('%H:%M')


	def update_sample_date(self, fname, type):
		mtime = os.path.getmtime(fname)
		dt = datetime.fromtimestamp(mtime)
		if type == 'cs': self.cs_date_entry.set_date(dt)
		if type == 'sv': self.sv_date_entry.set_date(dt)


	def _format_date(self, date_obj):
		if self.uset.date_format == 0:
			return date_obj.strftime('%m/%d/%Y')
		elif self.uset.date_format == 1:
			return date_obj.strftime('%d.%m.%Y')
		elif self.uset.date_format == 2:
			return date_obj.strftime('%Y-%m-%d')


	def _test_sv(self, type):
		_ = self.aset.translate
		if type == 'mx':
			msg = _("Analysis of combined sample, computation of AVQI/ABI, and voice quality profile will be omitted.")
		else:
			msg = _("Analysis will be incomplete.")
		if self.aset.sv_too_long:
			okay = askokcancel(
				_("Warning"),
				_("Vowel sample is too long (> 3 seconds).") + '\n\n' + msg,
				icon='warning'
				)
			return (0 if okay else -1)
		elif self.aset.sv_too_short:
			okay = askokcancel(
				_("Warning"),
				_("Vowel sample is too short (< 3 seconds).") + '\n\n' + msg,
				icon='warning'
				)
			return (0 if okay else -1)
		else:
			return 1


	def _clear_data(self):
		for tmpfile in self.aset.tmp_files:
			if os.path.exists(tmpfile):
				os.remove(tmpfile)



