# -*- coding: utf-8 -*-

#    VOXplot
#    
#    Copyright: Jörg Mayer
#    Email: joerg.mayer@lingphon.com
#    Web: https://voxplot.lingphon.com
#    
#    VOXplot is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#    
#    VOXplot is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#    
#    You should have received a copy of the GNU General Public
#    License along with VOXplot (cf. COPYING). If not, see
#    https://www.gnu.org/licenses/gpl-3.0.en.html



import tkinter as tk
from tkinter import ttk
from tkinter.messagebox import showerror, showinfo, showwarning



class Window(tk.Toplevel):
	'''corpus analysis

	:param parent: Object, reference to parent window
	:param ASet: Object, reference to AppSettings instance
	:param USet: Object, reference to UserSettings instance
	'''
	def __init__(self, parent, ASet, USet):
		super().__init__(parent)
		self.resizable(tk.FALSE,tk.FALSE)
		self.grab_set()
		self.attributes('-topmost', 1)
		self.protocol('WM_DELETE_WINDOW', self._close_pressed)
		self.aset = ASet
		self.uset = USet
		_ = self.aset.translate

		co_frame = ttk.Frame(self, padding=(10,10,10,10))
		co_frame.grid(column=0, row=0)
		self.columnconfigure(0, weight=0)
		self.columnconfigure(1, weight=0)
		self.rowconfigure(0, weight=0)

		ttk.Label(
			co_frame,
			text=_("VOXplot Corpus Analysis"),
			font='-weight bold'
			).grid(column=0, columnspan=2, row=1, pady=(0,0), sticky='w')


		self.co_folder_btn = ttk.Button(
			co_frame,
			text=_("Choose corpus folder"),
			command=self._co_folder_pressed
			)
		self.co_folder_btn.grid(column=0, columnspan=2, row=3, sticky='w', padx=(0,0), pady=(10,0))


		ttk.Label(
			co_frame,
			text=_("Recursive search") + ': '
			).grid(column=0, row=4, sticky='w', padx=(0,10), pady=(10,0))

		self.recursive = tk.BooleanVar()
		recursiveframe = ttk.Frame(co_frame)
		recursiveframe.grid(column=1, row=4, sticky='w')
		self.recuryes = ttk.Radiobutton(
			recursiveframe,
			text=_("Yes"),
			variable=self.recursive,
			value=True
			)
		self.recuryes.grid(column=0, row=0, pady=(10,0), padx=(0,10), sticky='w')
		self.recurno = ttk.Radiobutton(
			recursiveframe,
			text=_("No"),
			variable=self.recursive,
			value=False
			)
		self.recurno.grid(column=1, row=0, pady=(10,0), padx=(0,0), sticky='w')
		self.recuryes.invoke()


		ttk.Label(
			co_frame,
			text=_("Voice samples"),
			).grid(column=0, row=5, sticky='w', padx=(0,10), pady=(10,0))
		self.vsamples_cbox = ttk.Combobox(
			co_frame,
			# values=self.aset.gui_langs,
			width=25
			)
		self.vsamples_cbox['values'] = ('SV', 'CS', 'SV+CS')
		self.vsamples_cbox.current(0)
		self.vsamples_cbox.state(['readonly'])
		self.vsamples_cbox.grid(column=1, row=5, sticky='w', pady=(10,0))
		self.vsamples_cbox.bind('<<ComboboxSelected>>', self._vsamples_modified)




		# 	Suffix: wav or WAV
		# 	Select material: sv only/cs only/sv+cs
		# 	if sv+cs:
		# 	organization: single file/separate files
		# 	if separate files:
		# 	distinguishing pattern
		# 	if cs or sv+cs:
		# 	with voiceless/without voiceless - or automatic detection??





		btn_frame = ttk.Frame(co_frame)
		btn_frame.grid(column=1, row=200, pady=(20,0), sticky='e')
		btn_frame.columnconfigure(0, weight=1)
		ttk.Button(
			btn_frame,
			text=_("Cancel"),
			command=self._close_pressed
			).grid(column=0, row=0)

		self.bind('<KeyPress-Escape>', self._close_pressed)


	def _close_pressed(self, *args):
		self.destroy()


	def _co_folder_pressed(self, *args):
		pass


	def _vsamples_modified(self, event):
		_ = self.aset.translate

		### ..... ###

		showwarning(
			_("Warning"),
			_("You have choosen: ") + str(self.vsamples_cbox.current()),
			parent=self
			)
