# -*- coding: utf-8 -*-

#    VOXplot
#    
#    Copyright: Jörg Mayer
#    Email: joerg.mayer@lingphon.com
#    Web: https://voxplot.lingphon.com
#    
#    VOXplot is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#    
#    VOXplot is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#    
#    You should have received a copy of the GNU General Public
#    License along with VOXplot (cf. COPYING). If not, see
#    https://www.gnu.org/licenses/gpl-3.0.en.html



import tkinter as tk
from tkinter import ttk
from tkinter.messagebox import showerror, showwarning
import platform
import pickle
from datetime import datetime
from voxplot import helpers



class Window(tk.Toplevel):
	'''Edit user settings and display system information.

	:para parent: Object, reference to parent window
	:param ASet: Object, reference to AppSettings instance (~global variables)
	:param USet: Object, reference to UserSettings instance
	'''
	def __init__(self, parent, ASet, USet):
		super().__init__(parent)
		self.resizable(tk.FALSE,tk.FALSE)
		self.grab_set()
		# self.attributes('-topmost', 1)
		self.protocol('WM_DELETE_WINDOW', self.cancel_pressed)
		self.columnconfigure(0, weight=1)
		self.rowconfigure(0, weight=1)
		self.aset = ASet
		self.uset = USet
		_ = self.aset.translate


		### frames
		mainframe = ttk.Frame(self, padding=(10,10,10,10))
		mainframe.columnconfigure(0, weight=1)
		mainframe.columnconfigure(1, weight=1)
		mainframe.rowconfigure(0, weight=1)
		mainframe.grid(column=0, row=0)

		set_frame = ttk.Frame(mainframe)
		set_frame.grid(column=0, row=0, sticky='nsw', padx=(10,15), pady=10)
		set_frame.rowconfigure(0, weight=0)
		set_frame.rowconfigure(1, weight=0)
		set_frame.rowconfigure(2, weight=0)
		set_frame.rowconfigure(3, weight=0)
		set_frame.rowconfigure(4, weight=0)
		set_frame.rowconfigure(5, weight=0)
		set_frame.rowconfigure(6, weight=0)
		set_frame.rowconfigure(7, weight=0)
		set_frame.rowconfigure(8, weight=0)
		set_frame.rowconfigure(9, weight=0)
		set_frame.rowconfigure(10, weight=0)
		set_frame.rowconfigure(11, weight=0)
		set_frame.rowconfigure(12, weight=0)
		set_frame.rowconfigure(13, weight=0)
		set_frame.rowconfigure(14, weight=0)
		set_frame.rowconfigure(15, weight=0)
		set_frame.rowconfigure(16, weight=0)
		set_frame.rowconfigure(20, weight=1)

		info_frame = ttk.Frame(mainframe)
		info_frame.grid(column=1, row=0, sticky='nswe', padx=(15,10), pady=10)
		info_frame.rowconfigure(0, weight=0)
		info_frame.rowconfigure(1, weight=1)
		info_frame.columnconfigure(0, weight=1)

		self.aset.style.configure('Head1.TLabel', font=f'{self.aset.font_family} 18 bold')
		self.aset.style.configure('Head2.TLabel', font=f'{self.aset.font_family} {self.aset.font_size} bold')


		### left column
		ttk.Label(
			set_frame,
			text=_("Settings"),
			# font='-size 18 -weight bold',
			style='Head1.TLabel',
			).grid(column=0, row=0, columnspan=2, sticky='w', pady=(0,10))

		# default recording duration
		ttk.Label(
			set_frame,
			text=_("Default recording duration"),
			# font='-weight bold',
			style='Head2.TLabel',
			).grid(column=0, row=1, columnspan=2, sticky='w', pady=(0,5))

		ttk.Label(
			set_frame,
			text=_("continuous speech"),
			).grid(column=0, row=2, sticky='ne', padx=(10,10), pady=(0,5))
		self.set_cs_recdur = tk.StringVar()
		cs_spinbox = ttk.Spinbox(
			set_frame,
			from_=5,
			to=30,
			width=3,
			textvariable=self.set_cs_recdur
			)
		self.set_cs_recdur.set(str(self.uset.rec_duration['cs']))
		cs_spinbox.grid(column=1, row=2, sticky='sw', pady=(0,5))
		cs_spinbox.state(['readonly'])

		ttk.Label(
			set_frame,
			text=_("sustained vowel"),
			).grid(column=0, row=3, sticky='ne', padx=(10,10), pady=(0,5))
		self.set_sv_recdur = tk.StringVar()
		sv_spinbox = ttk.Spinbox(
			set_frame,
			from_=4,
			to=15,
			width=3,
			textvariable=self.set_sv_recdur
			)
		self.set_sv_recdur.set(str(self.uset.rec_duration['sv']))
		sv_spinbox.grid(column=1, row=3, sticky='sw', pady=(0,5))
		sv_spinbox.state(['readonly'])

		# default examiner
		ttk.Label(
			set_frame,
			text=_("Default examiner"),
			# font='-weight bold',
			style='Head2.TLabel',
			).grid(column=0, row=4, columnspan=2, sticky='w', pady=(10,5))

		ttk.Label(
			set_frame,
			text=_("examiner"),
			).grid(column=0, row=5, sticky='ne', padx=(10,10))
		self.exam_n = tk.StringVar()
		examiner_entry = ttk.Entry(
			set_frame,
			width=25,
			textvariable=self.exam_n
			)
		self.exam_n.set(self.uset.examiner)
		examiner_entry.grid(column=1, row=5, sticky='sw')

		# language settings
		ttk.Label(
			set_frame,
			text=_("Language Settings"),
			# font='-weight bold',
			style='Head2.TLabel',
			).grid(column=0, row=6, columnspan=2, sticky='w', pady=(10,5))

		ttk.Label(
			set_frame,
			text=_("Interface language"),
			).grid(column=0, row=7, sticky='ne', padx=(10,10), pady=(0,5))
		self.ilang_cbox = ttk.Combobox(
			set_frame,
			width=25,
			values=self.aset.gui_langs
			)
		self.ilang_cbox.current(self.uset.i_lang)
		self.ilang_cbox.state(['readonly'])
		self.ilang_cbox.grid(column=1, row=7, sticky='sw', pady=(0,5))
		self.ilang_cbox.bind('<<ComboboxSelected>>', self.ilang_modified)

		ttk.Label(
			set_frame,
			text=_("Analysis language"),
			).grid(column=0, row=8, sticky='ne', padx=(10,10), pady=(0,5))
		self.alang_cbox = ttk.Combobox(
			set_frame,
			width=25,
			values=self.aset.analysis_langs
			)
		self.alang_cbox.current(self.uset.a_lang)
		self.alang_cbox.state(['readonly'])
		self.alang_cbox.grid(column=1, row=8, sticky='sw', pady=(0,5))

		# formats
		ttk.Label(
			set_frame,
			text=_("Formats"),
			# font='-weight bold',
			style='Head2.TLabel',
			).grid(column=0, row=9, columnspan=2, sticky='w', pady=(10,5))

		ttk.Label(
			set_frame,
			text=_("CSV field delimiter"),
			).grid(column=0, row=10, sticky='ne', padx=(10,10), pady=(0,5))
		self.csv_field_cbox = ttk.Combobox(
			set_frame,
			width=25,
			values=self.aset.csv_field_delim_list
			)
		self.csv_field_cbox.current(self.uset.csv_field_delim)
		self.csv_field_cbox.state(['readonly'])
		self.csv_field_cbox.grid(column=1, row=10, sticky='sw', pady=(0,5))

		ttk.Label(
			set_frame,
			text=_("CSV decimal separator"),
			).grid(column=0, row=11, sticky='ne', padx=(10,10), pady=(0,5))
		self.csv_decimal_cbox = ttk.Combobox(
			set_frame,
			width=25,
			values=self.aset.csv_decimal_sep_list
			)
		self.csv_decimal_cbox.current(self.uset.csv_decimal_sep)
		self.csv_decimal_cbox.state(['readonly'])
		self.csv_decimal_cbox.grid(column=1, row=11, sticky='sw', pady=(0,5))

		ttk.Label(
			set_frame,
			text=_("Date format"),
			).grid(column=0, row=12, sticky='ne', padx=(10,10), pady=(0,5))
		self.date_format_cbox = ttk.Combobox(
			set_frame,
			width=25,
			values=self.aset.date_format_list
			)
		self.date_format_cbox.current(self.uset.date_format)
		self.date_format_cbox.state(['readonly'])
		self.date_format_cbox.grid(column=1, row=12, sticky='sw', pady=(0,5))

		ttk.Label(
			set_frame,
			text=_("Time format"),
			).grid(column=0, row=13, sticky='ne', padx=(10,10), pady=(0,5))
		self.time_format_cbox = ttk.Combobox(
			set_frame,
			width=25,
			values=self.aset.time_format_list
			)
		self.time_format_cbox.current(self.uset.time_format)
		self.time_format_cbox.state(['readonly'])
		self.time_format_cbox.grid(column=1, row=13, sticky='sw', pady=(0,5))

		# real-time signal plot
		ttk.Label(
			set_frame,
			text=_("Real-Time Signal Plot"),
			# font='-weight bold',
			style='Head2.TLabel',
			).grid(column=0, row=14, columnspan=2, sticky='w', pady=(10,5))
		ttk.Label(
			set_frame,
			text=_("Switch off in case of recording discontinuities."),
			).grid(column=0, row=15, columnspan=2, sticky='w', pady=(0,5))
		self.liveplot = tk.BooleanVar()
		self.liveplot.set(self.uset.live_plot)
		ttk.Checkbutton(
			set_frame,
			text=_("real-time signal plot"),
			variable=self.liveplot,
			).grid(column=1, row=16, sticky='sw', pady=(0,5))

		ttk.Button(
			set_frame,
			text=_("Cancel"),
			command=self.cancel_pressed
			).grid(column=0, row=20, sticky='sw', pady=(20,0))
		ttk.Button(
			set_frame,
			text=_("Save"),
			command=self.save_pressed
			).grid(column=1, row=20, sticky='se', pady=(20,0))


		### right column
		# recording device
		ttk.Label(
			info_frame,
			text=_("Active Recording Device"),
			# font='-size 18 -weight bold',
			style='Head1.TLabel',
			).grid(column=0, row=0, sticky='w', pady=(0,10))
		self.audio_text = tk.Text(
			info_frame,
			width=50,
			height=10,
			wrap='word',
			font='TkDefaultFont',
			relief='flat',
			padx=10,
			pady=10,
			background='#ffffff'
			)
		self.audio_text.grid(column=0, row=1, sticky='nswe')
		audio_info = self.get_audio_info()
		self.audio_text.insert('1.0', audio_info)
		self.audio_text['state'] = 'disabled'

		# system information
		ttk.Label(
			info_frame,
			text=_("System Information"),
			# font='-size 18 -weight bold',
			style='Head1.TLabel',
			).grid(column=0, row=2, sticky='w', pady=(20,10))
		info_text = tk.Text(
			info_frame,
			width=50,
			height=8,
			wrap='word',
			font='TkDefaultFont',
			relief='flat',
			padx=10,
			pady=10,
			background='#ffffff'
			)
		info_text.grid(column=0, row=3, sticky='nswe')
		info_text.insert(
			'1.0',
			"System: " + platform.system() + "\n"
			"Release: " + platform.release() + "\n"
			"Version: " + platform.version() + "\n"
			"Arch: " + platform.machine() + "\n"
			"Praat: " + self.aset.praat_exe
			# "\n\nappdir: " + APP_DIR + "\n"
			# "homedir: " + HOME_DIR + "\n"
			# "workdir: " + WORKING_DIR + "\n"
			)
		info_text['state'] = 'disabled'


		### bindings
		self.bind('<KeyPress-Escape>', self.cancel_pressed)
		self.audio_text_status = 'info'
		self.bind('<Control-KeyPress-i>', self.audio_debug)


	def ilang_modified(self, event):
		if self.uset.i_lang != self.ilang_cbox.current():
			self.uset.i_lang = self.ilang_cbox.current()
			self.aset.translate = helpers.set_gui_language(self.aset.gui_langcodes, self.aset.translations, self.uset)
			_ = self.aset.translate
			showwarning(
				_("Warning"),
				_("To activate a new interface language save settings, quit, and restart VOXplot."),
				parent=self
				)


	def save_pressed(self, *args):
		app_win = self.aset.app_win
		_ = self.aset.translate

		# update user settings
		self.uset.rec_duration['cs'] = int(self.set_cs_recdur.get())
		self.uset.rec_duration['sv'] = int(self.set_sv_recdur.get())
		self.uset.examiner = self.exam_n.get()
		self.uset.i_lang = self.ilang_cbox.current()
		self.uset.a_lang = self.alang_cbox.current()
		app_win.exam_name.set(self.uset.examiner)
		app_win.alanginfo_link.config(text = _("Analysis language") + ': ' + self.aset.analysis_langs[self.uset.a_lang])
		cfd = self.csv_field_cbox.current()
		cds = self.csv_decimal_cbox.current()
		if cfd == 0 and cds == 1:
			showwarning(
				_("Warning"),
				_("Field delimiter and decimal separator must differ. You selected 'comma' for both"),
				parent=self
				)
			return
		self.uset.csv_field_delim = cfd
		self.uset.csv_decimal_sep = cds
		self.uset.date_format = self.date_format_cbox.current()
		app_win.set_date_format(app_win.exam_date)
		app_win.set_date_format(app_win.cs_date_entry)
		app_win.set_date_format(app_win.sv_date_entry)
		self.uset.time_format = self.time_format_cbox.current()
		app_win.exam_time.set(app_win.format_time(datetime.today()))
		self.uset.live_plot = self.liveplot.get()
		# set selected analysis language
		self.aset.analysis_langs, self.aset.cs_text, self.aset.norm_values = helpers.set_ana_language(self.aset.alang_data, self.alang_cbox.current())

		# persistence: write settings to pickle file
		settings_data = {
			'rdcs' : self.uset.rec_duration['cs'],
			'rdsv' : self.uset.rec_duration['sv'],
			'ex' : self.uset.examiner,
			'ilang' : self.uset.i_lang,
			'alang' : self.uset.a_lang,
			'csvf' : self.uset.csv_field_delim,
			'csvd' : self.uset.csv_decimal_sep,
			'datef' : self.uset.date_format,
			'timef' : self.uset.time_format,
			'lplot' : self.uset.live_plot,
			}
		try:
			pickle.dump(settings_data, open(self.aset.settings_file, 'wb'))
		except Exception as e:
			showerror(
				_("Save settings failed"),
				_("Can't write to settings file. Next session will fall back to default settings.") + "\n\n" + e.args[0],
				parent=self
				)
		self.destroy()


	def cancel_pressed(self, *args):
		self.destroy()


	def audio_debug(self, *args):
		self.audio_text.configure(state='normal')
		self.audio_text.delete('1.0', 'end')
		if self.audio_text_status == 'info':
			self.audio_text_status = 'debug'
			audio_debug_info = helpers.input_devices_debug()
			self.audio_text.insert('1.0', audio_debug_info)
		else:
			self.audio_text_status = 'info'
			audio_info = self.get_audio_info()
			self.audio_text.insert('1.0', audio_info)
			self.audio_text.configure(state='disabled')


	def get_audio_info(self):
		_ = self.aset.translate
		ida, device_info = helpers.input_device_test(self.aset)
		parsed = _("Device name") + ":   " + device_info['name'] + '\n' +\
			_("Max input channels") + ":   " + device_info['maxInputChannels'] + '\n' +\
			_("Sampling rate") + ":   " + device_info['defaultSampleRate'] + ' Hz\n' +\
			'\n' + _("To change the active recording device: (1) Quit VOXplot, (2) select the desired device using the tools provided by your operating system, and (3) restart VOXplot.")
		return parsed





